from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from drf_yasg import openapi
from drf_yasg.views import get_schema_view

from ...core.permissions import is_staff
from ..views import local_space_available, CURRENT_DIR, logger


class StatsView(APIView):
    """
    Return instance stats
    """

    permission_classes = (AllowAny,)

    def get(self, request):
        try:
            #stats = os.statvfs(CURRENT_DIR)
            free_space_mb = int(local_space_available(CURRENT_DIR) / (1024 * 1024))
            # free_space_mb = int(
            #     (stats.f_bavail * stats.f_frsize) / (1024 * 1024))

            logger.info(
                'Free space (MB): {}.'.format(free_space_mb))

            if free_space_mb > 200:
                health_status = 'green'
            else:
                if free_space_mb < 100:
                    health_status = 'yellow'
                else:
                    health_status = 'red'

            data = {
                'status': health_status,
                'meta': str(request.META.items())
            }

            if is_staff(request.user):
                data['free_space_mb'] = free_space_mb

            return Response(data, status.HTTP_200_OK)

        except Exception as e:
            err_msg = str(e)
            logger.exception('Error getting health {}'.format(err_msg))
            return Response(err_msg, status.HTTP_500_INTERNAL_SERVER_ERROR)


schema_view = get_schema_view(
   openapi.Info(
      title="Django SSO App",
      default_version='v1',
   ),
   public=True,
   permission_classes=(AllowAny,),
)



import logging

from django.contrib.auth.models import Group as GroupModel
from django.db.models.signals import m2m_changed
from django.dispatch import receiver

from ....permissions import is_staff
from ..models import Group
from ...profiles.models import Profile

logger = logging.getLogger('django_sso_app.core.apps.groups.signals')


@receiver(m2m_changed)
def signal_handler_when_user_is_added_or_removed_from_group(action, instance, pk_set, model, **kwargs):
    if model == Group and instance.__class__ == Profile:
        user = instance.user
        profile = instance

        if not user.is_superuser:
            logger.info('Groups signal')

            is_loaddata = getattr(profile, '__django_sso_app__loaddata', False)
            is_creating = getattr(user, '__django_sso_app__creating', False)

            must_update_rev = (not is_loaddata) and (not is_creating)

            groups_updated = False

            if action == 'pre_add':
                groups_updated = True
                for pk in pk_set:
                    _group = Group.objects.get(id=pk)
                    logger.info('Profile "{}" entered group "{}"'.format(profile, _group))

            elif action == 'pre_remove':
                groups_updated = True
                for pk in pk_set:
                    _group = Group.objects.get(id=pk)
                    logger.info('Profile "{}" exited from group "{}"'.format(profile, _group))

            # updating rev

            if groups_updated and must_update_rev:
                profile.update_rev(True)



# profile filed update updates rev

from django.contrib.auth import get_user_model

from django_sso_app.backend.tests.factories import UserTestCase

User = get_user_model()


class ProfileTestCase(UserTestCase):

    def test_profile_field_update_updates_rev(self):
        """
        User update updates user revision
        """

        profile_rev = self.profile.sso_rev

        self.profile.country = 'IT'
        self.profile.save()

        self.profile.refresh_from_db()

        self.assertEqual(self.profile.sso_rev, profile_rev + 1)



import logging

from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save
from django.contrib.auth import get_user_model
from django.contrib.auth.signals import user_logged_in

from ....permissions import is_staff
from ..utils import get_or_create_user_profile
from ..models import Profile

logger = logging.getLogger('django_sso_app.core.apps.profiles.signals')

User = get_user_model()


@receiver(pre_save, sender=Profile)
def profile_pre_updated(sender, instance, **kwargs):
    """
    Profile model has been updated, updating rev
    """
    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        return

    if not instance._state.adding:
        user = instance.user

        if not user.is_superuser:
            if getattr(user, '__django_sso_app__apigateway_update', False):
                logger.debug('Profile has been updted by api gateway, skipping rev update')
            elif getattr(user, '__django_sso_app__creating', False):
                logger.debug('Created, skipping rev update')
            else:
                logger.debug('Profile model has been updated, updating rev')
                instance.update_rev(False)


@receiver(post_save, sender=Profile)
def profile_updated(sender, instance, created, **kwargs):
    """
    Profile model has been updated, updating rev
    """
    profile = instance

    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        setattr(profile.user, '__django_sso_app__loaddata', True)
        return

    if not created:  # if instance.pk:
        logger.debug('Profile model has been updated, refreshing instance')
        profile.refresh_from_db()

# user

@receiver(post_save, sender=User)
def create_update_user_profile(sender, instance, created, **kwargs):
    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        # setting property flag
        setattr(instance, '__django_sso_app__loaddata', True)
        return

    user = instance

    if not user.is_superuser:

        if created:
            logger.debug('user created, creating profile')

            profile = get_or_create_user_profile(user, Profile, commit=True)

            logger.debug('new profile created "{}"'.format(profile))

            # refreshing user instance
            user.sso_app_profile = profile

        else:
            profile = get_or_create_user_profile(user, Profile, commit=False)
            update_rev = True

            # replaced by '__user_loggin_in'
            # if user.previous_serialized_as_string == user.serialized_as_string:
            if getattr(user, '__django_sso_app__creating', False):
                update_rev = False
            if getattr(user, '__django_sso_app__user_loggin_in', False):
                update_rev = False
            if user.password_has_been_hardened:
                update_rev = False
            if getattr(user, '__django_sso_app__email_updated', False):
                update_rev = True
            if getattr(user, '__django_sso_app__password_updated', False):
                update_rev = True

            if update_rev:
                logger.info('Update rev by User signal while user fields have been updated')
                profile.update_rev(True)  # updating rev


@receiver(user_logged_in)
def post_user_login(**kwargs):
    """
    Post login profile creation safety
    :param kwargs:
    :return:
    """
    user = kwargs['user']

    if not user.is_superuser:
        _profile = get_or_create_user_profile(user, Profile)
        # setattr(user, 'sso_app_profile', _profile)

        logger.debug('Profile, "{}" LOGGED IN!!!'.format(_profile))
